@@ -17,6 +17,7 @@ elif isosx:
 class PluginNotFound(ValueError):
     pass
 
+
 class InvalidPlugin(ValueError):
     pass
 
@@ -84,8 +85,10 @@ class Plugin(object):  # {{{
 
     def initialize(self):
         '''
-        Called once when calibre plugins are initialized. Plugins are re-initialized
-        every time a new plugin is added.
+        Called once when calibre plugins are initialized.  Plugins are
+        re-initialized every time a new plugin is added. Also note that if the
+        plugin is run in a worker process, such as for adding books, then the
+        plugin will be initialized for every new worker process.
 
         Perform any plugin specific initialization here, such as extracting
         resources from the plugin zip file. The path to the zip file is
@@ -177,8 +180,7 @@ class Plugin(object):  # {{{
             help_text = self.customization_help(gui=True)
             help_text = QLabel(help_text, config_dialog)
             help_text.setWordWrap(True)
-            help_text.setTextInteractionFlags(Qt.LinksAccessibleByMouse
-                    | Qt.LinksAccessibleByKeyboard)
+            help_text.setTextInteractionFlags(Qt.LinksAccessibleByMouse | Qt.LinksAccessibleByKeyboard)
             help_text.setOpenExternalLinks(True)
             v.addWidget(help_text)
             sc = plugin_customization(self)
@@ -311,12 +313,14 @@ class Plugin(object):  # {{{
 
 # }}}
 
+
 class FileTypePlugin(Plugin):  # {{{
     '''
     A plugin that is associated with a particular set of file types.
     '''
 
     #: Set of file types for which this plugin should be run.
+    #: Use '*' for all file types.
     #: For example: ``{'lit', 'mobi', 'prc'}``
     file_types     = set()
 
@@ -325,7 +329,8 @@ class FileTypePlugin(Plugin):  # {{{
     on_import      = False
 
     #: If True, this plugin is run after books are added
-    #: to the database
+    #: to the database. In this case the postimport and postadd
+    #: methods of the plugin are called.
     on_postimport  = False
 
     #: If True, this plugin is run just before a conversion
@@ -345,7 +350,9 @@ class FileTypePlugin(Plugin):  # {{{
         modified ebook. If no modifications are needed, it should
         return the path to the original ebook. If an error is encountered
         it should raise an Exception. The default implementation
-        simply return the path to the original ebook.
+        simply return the path to the original ebook. Note that the path to
+        the original file (before any file type plugins are run, is available as
+        self.original_path_to_file).
 
         The modified ebook file should be created with the
         :meth:`temporary_file` method.
@@ -359,16 +366,38 @@ class FileTypePlugin(Plugin):  # {{{
 
     def postimport(self, book_id, book_format, db):
         '''
-        Called post import, i.e., after the book file has been added to the database.
+        Called post import, i.e., after the book file has been added to the database. Note that
+        this is different from :meth:`postadd` which is called when the book record is created for
+        the first time. This method is called whenever a new file is added to a book record. It is
+        useful for modifying the book record based on the contents of the newly added file.
 
         :param book_id: Database id of the added book.
         :param book_format: The file type of the book that was added.
-                :param db: Library database.
+        :param db: Library database.
+        '''
+        pass  # Default implementation does nothing
+
+    def postadd(self, book_id, fmt_map, db):
+        '''
+        Called post add, i.e. after a book has been added to the db. Note that
+        this is different from :meth:`postimport`, which is called after a single book file
+        has been added to a book. postadd() is called only when an entire book record
+        with possibly more than one book file has been created for the first time.
+        This is useful if you wish to modify the book record in the database when the
+        book is first added to calibre.
+
+        :param book_id: Database id of the added book.
+        :param fmt_map: Map of file format to path from which the file format
+            was added. Note that this might or might not point to an actual
+            existing file, as sometimes files are added as streams. In which case
+            it might be a dummy value or a non-existent path.
+        :param db: Library database
         '''
         pass  # Default implementation does nothing
 
 # }}}
 
+
 class MetadataReaderPlugin(Plugin):  # {{{
     '''
     A plugin that implements reading metadata from a set of file types.
@@ -392,13 +421,15 @@ class MetadataReaderPlugin(Plugin):  # {{{
         Return metadata for the file represented by stream (a file like object
         that supports reading). Raise an exception when there is an error
         with the input data.
+
         :param type: The type of file. Guaranteed to be one of the entries
-        in :attr:`file_types`.
+            in :attr:`file_types`.
         :return: A :class:`calibre.ebooks.metadata.book.Metadata` object
         '''
         return None
 # }}}
 
+
 class MetadataWriterPlugin(Plugin):  # {{{
     '''
     A plugin that implements reading metadata from a set of file types.
@@ -422,14 +453,16 @@ class MetadataWriterPlugin(Plugin):  # {{{
         Set metadata for the file represented by stream (a file like object
         that supports reading). Raise an exception when there is an error
         with the input data.
+
         :param type: The type of file. Guaranteed to be one of the entries
-        in :attr:`file_types`.
+            in :attr:`file_types`.
         :param mi: A :class:`calibre.ebooks.metadata.book.Metadata` object
         '''
         pass
 
 # }}}
 
+
 class CatalogPlugin(Plugin):  # {{{
     '''
     A plugin that implements a catalog generator.
@@ -557,6 +590,7 @@ class CatalogPlugin(Plugin):  # {{{
 
 # }}}
 
+
 class InterfaceActionBase(Plugin):  # {{{
 
     supported_platforms = ['windows', 'osx', 'linux']
@@ -584,6 +618,7 @@ class InterfaceActionBase(Plugin):  # {{{
 
 # }}}
 
+
 class PreferencesPlugin(Plugin):  # {{{
 
     '''
@@ -643,6 +678,7 @@ class PreferencesPlugin(Plugin):  # {{{
 
 # }}}
 
+
 class StoreBase(Plugin):  # {{{
 
     supported_platforms = ['windows', 'osx', 'linux']
@@ -692,6 +728,7 @@ class StoreBase(Plugin):  # {{{
 
 # }}}
 
+
 class ViewerPlugin(Plugin):  # {{{
 
     type = _('Viewer')
@@ -751,6 +788,7 @@ class ViewerPlugin(Plugin):  # {{{
 
 # }}}
 
+
 class EditBookToolPlugin(Plugin):  # {{{
 
     type = _('Edit Book Tool')
@@ -758,3 +796,25 @@ class EditBookToolPlugin(Plugin):  # {{{
 
 # }}}
 
+
+class LibraryClosedPlugin(Plugin):  # {{{
+    '''
+    LibraryClosedPlugins are run when a library is closed, either at shutdown,
+    when the library is changed, or when a library is used in some other way.
+    At the moment these plugins won't be called by the CLI functions.
+    '''
+    type = _('Library Closed')
+
+    # minimum version 2.54 because that is when support was added
+    minimum_calibre_version = (2, 54, 0)
+
+    def run(self, db):
+        '''
+        The db will be a reference to the new_api (db.cache.py).
+
+        The plugin must run to completion. It must not use the GUI, threads, or
+        any signals.
+        '''
+        raise NotImplementedError('LibraryClosedPlugin '
+                'run method must be overridden in subclass')
+# }}}
