@@ -25,6 +25,7 @@ from calibre.utils.icu import numeric_sort_key as sort_key
 
 ENTRY_ROLE = Qt.UserRole
 
+
 def pixmap_to_data(pixmap):
     ba = QByteArray()
     buf = QBuffer(ba)
@@ -32,6 +33,7 @@ def pixmap_to_data(pixmap):
     pixmap.save(buf, 'PNG')
     return bytearray(ba.data())
 
+
 def run_program(entry, path, parent):
     import subprocess
     cmdline = entry_to_cmdline(entry, path)
@@ -48,6 +50,7 @@ def run_program(entry, path, parent):
     t.daemon = True
     t.start()
 
+
 def entry_to_icon_text(entry, only_text=False):
     if only_text:
         return entry.get('name', entry.get('Name')) or _('Unknown')
@@ -117,7 +120,8 @@ if iswindows:
         return cmdline.replace('%1', qpath)
 
     del run_program
-    def run_program(entry, path, parent):
+
+    def run_program(entry, path, parent):  # noqa
         cmdline = entry_to_cmdline(entry, path)
         print('Running Open With commandline:', repr(entry['cmdline']), ' |==> ', repr(cmdline))
         try:
@@ -184,6 +188,8 @@ else:
 
     def entry_to_item(entry, parent):
         icon_path = entry.get('Icon') or I('blank.png')
+        if not isinstance(icon_path, basestring):
+            icon_path = I('blank.png')
         ans = QListWidgetItem(QIcon(icon_path), entry.get('Name') or _('Unknown'), parent)
         ans.setData(ENTRY_ROLE, entry)
         comment = (entry.get('Comment') or '')
@@ -215,6 +221,7 @@ else:
         return entry
 # }}}
 
+
 class ChooseProgram(Dialog):  # {{{
 
     found = pyqtSignal()
@@ -237,7 +244,8 @@ class ChooseProgram(Dialog):  # {{{
         self.pi = pi = ProgressIndicator(self, 256)
         l.addStretch(1), l.addWidget(pi, alignment=Qt.AlignHCenter), l.addSpacing(10)
         w.la = la = QLabel(_('Gathering data, please wait...'))
-        la.setStyleSheet('QLabel { font-size: 30pt; font-weight: bold }')
+        f = la.font()
+        f.setBold(True), f.setPointSize(28), la.setFont(f)
         l.addWidget(la, alignment=Qt.AlignHCenter), l.addStretch(1)
         s.addWidget(w)
 
@@ -294,6 +302,7 @@ class ChooseProgram(Dialog):  # {{{
 
 oprefs.defaults['entries'] = {}
 
+
 def choose_program(file_type='jpeg', parent=None, prefs=oprefs):
     oft = file_type = file_type.lower()
     file_type = {'cover_image':'jpeg'}.get(oft, oft)
@@ -312,6 +321,7 @@ def choose_program(file_type='jpeg', parent=None, prefs=oprefs):
         register_keyboard_shortcuts(finalize=True)
     return entry
 
+
 def populate_menu(menu, receiver, file_type):
     file_type = file_type.lower()
     for entry in oprefs['entries'].get(file_type, ()):
@@ -327,6 +337,7 @@ def populate_menu(menu, receiver, file_type):
 
 # }}}
 
+
 class EditPrograms(Dialog):  # {{{
 
     def __init__(self, file_type='jpeg', parent=None):
@@ -391,6 +402,7 @@ class EditPrograms(Dialog):  # {{{
         oprefs['entries'][self.file_type] = entries
         oprefs['entries'] = oprefs['entries']
 
+
 def edit_programs(file_type, parent):
     d = EditPrograms(file_type, parent)
     d.exec_()
@@ -398,6 +410,7 @@ def edit_programs(file_type, parent):
 
 registered_shortcuts = {}
 
+
 def register_keyboard_shortcuts(gui=None, finalize=False):
     if gui is None:
         from calibre.gui2.ui import get_gui
